package de.hs_mannheim.informatik.pr2projekt.gui;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.GridLayout;
import java.io.FileNotFoundException;
import java.util.EmptyStackException;
import java.util.Stack;

import javax.swing.JButton;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JTextField;

import de.hs_mannheim.informatik.pr2projekt.domain.HitoriMain;

public class GameGUI {
	
    public static void paintGame(String[] filepath, JButton[][] buttons, String[][] colors, Stack<String> madeMoves, String[][] data){
        JFrame frame = new JFrame();
        int num = buttons.length;
        JPanel gameGrid = new JPanel(new GridLayout(num,num,0,0));
        for(int i=0;i<num;i++){
            for(int j=0;j<num;j++){
                JButton b = buttons[i][j];
                b.setOpaque(true);
                b.setContentAreaFilled(true);
                b.setBorderPainted(false);
                b.setFocusPainted(false);
                b.setBackground(Color.WHITE);
                gameGrid.add(b);
                String[] pos = HitoriMain.getCords(i,j); 
                b.addActionListener(e -> {paintButton(b, pos, colors , madeMoves);});
            }
        }
        JPanel mainPanel = new JPanel(new BorderLayout());
        mainPanel.add(gameGrid, BorderLayout.CENTER);
        JPanel buttonGrid = new JPanel(new GridLayout(1,4,10,10));
        JButton b0 = new JButton("Aufgeben");
        b0.addActionListener(e -> {
            frame.dispose();
            MenuGUI.getPath(filepath);
        });
        JButton b1 = new JButton("Zurück");
        b1.addActionListener(e -> {HitoriMain.backOneStep(madeMoves, buttons, colors, gameGrid);});
        JButton b2 = new JButton("Zurücksetzen");
        b2.addActionListener(e -> {HitoriMain.totalResetButton(buttons, colors, madeMoves, data);});
        JButton b3 = new JButton("Abgeben");
        String path = filepath[0];
        b3.addActionListener(e -> {
            try{
            boolean levelFinished = HitoriMain.abgabeMöglich(path, data, colors);
            if(levelFinished == true){
                GameGUI.finish(filepath, path, data, colors);
                frame.dispose();
            } else {
                JLabel text1 = new JLabel("Abgabe nicht richtig!");
                mainPanel.add(text1, BorderLayout.NORTH);
                mainPanel.revalidate();
                mainPanel.repaint();
            }
            } catch (FileNotFoundException i) {
                System.out.println("File not found: " + i.getMessage());
            }
        });
        buttonGrid.add(b0);buttonGrid.add(b1);buttonGrid.add(b2);buttonGrid.add(b3);
        mainPanel.add(buttonGrid, BorderLayout.SOUTH);
        mainPanel.setVisible(true);
        frame.add(mainPanel);
        frame.setVisible(true);
        frame.setSize(600,600);
    }
    
    public static void paintButton(JButton b, String[] pos, String[][] colors, Stack<String> madeMoves){
        int i = Integer.parseInt(pos[0]);
        int j = Integer.parseInt(pos[1]);
        String col = colors[Integer.parseInt(pos[0])][Integer.parseInt(pos[1])];
        if(col.endsWith("W")){
            b.setOpaque(true);
            b.setForeground(Color.BLACK);
            b.setContentAreaFilled(true);
            b.setBorderPainted(false);
            b.setFocusPainted(false);
            b.setBackground(Color.lightGray);
            colors[i][j] += "G";
            String logEntrance = i+"."+j+"."+"G";
            madeMoves.push(logEntrance);
        }
        if(col.endsWith("G")){
            b.setOpaque(true);
            b.setForeground(Color.WHITE);
            b.setContentAreaFilled(true);
            b.setBorderPainted(false);
            b.setFocusPainted(false);
            b.setBackground(Color.BLACK);
            colors[i][j] += "B";
            String logEntrance = i+"."+j+"."+"B";
            madeMoves.push(logEntrance);
        }
        if(col.endsWith("B")){
            b.setOpaque(true);
            b.setForeground(Color.BLACK);
            b.setContentAreaFilled(true);
            b.setBorderPainted(false);
            b.setFocusPainted(false);
            b.setBackground(Color.WHITE);
            colors[i][j] += "W";
            String logEntrance = i+"."+j+"."+"W";
            madeMoves.push(logEntrance);
        }
    	}
        
        public static void backOneStep(Stack<String> movesMade, JButton[][] buttons, String[][] colors, JPanel grid){
            try {
                String move = movesMade.pop();
                String[] line = move.split("\\.");
                String y = line[0];
                String x = line[1];
                String color = line[2];
                if(color.equals("W")){
                    int i = Integer.parseInt(y);
                    int j = Integer.parseInt(x);
                    JButton b0 = buttons[i][j];
                    b0.setOpaque(true);
                    b0.setForeground(Color.WHITE);
                    b0.setContentAreaFilled(true);
                    b0.setBorderPainted(false);
                    b0.setFocusPainted(false);
                    b0.setBackground(Color.BLACK);
                    buttons[i][j] = b0;
                    String str = colors[i][j]; 
                    String str0 = str.substring(0, str.length() - 1);
                    colors[i][j] = str0;
                    grid.repaint();
                    gridUpdate(grid, buttons);
                }else if(color.equals("G")){
                    int i = Integer.parseInt(y);
                    int j = Integer.parseInt(x);
                    JButton b0 = buttons[i][j];
                    b0.setOpaque(true);
                    b0.setForeground(Color.BLACK);
                    b0.setContentAreaFilled(true);
                    b0.setBorderPainted(false);
                    b0.setFocusPainted(false);
                    b0.setBackground(Color.WHITE);
                    buttons[i][j] = b0;
                    String str = colors[i][j]; 
                    String str0 = str.substring(0, str.length() - 1);
                    colors[i][j] = str0;
                    gridUpdate(grid, buttons);
                } else if(color.equals("B")){
                    int i = Integer.parseInt(y);
                    int j = Integer.parseInt(x);
                    JButton b0 = buttons[i][j];
                    b0.setOpaque(true);
                    b0.setForeground(Color.BLACK);
                    b0.setContentAreaFilled(true);
                    b0.setBorderPainted(false);
                    b0.setFocusPainted(false);
                    b0.setBackground(Color.lightGray);
                    buttons[i][j] = b0;
                    String str = colors[i][j]; 
                    String str0 = str.substring(0, str.length() - 1);
                    colors[i][j] = str0;
                    gridUpdate(grid, buttons);
                } 
            } catch(EmptyStackException e) {}  
        }
        
        public static void gridUpdate(JPanel grid, JButton[][] buttons){
            grid.removeAll();
            grid.repaint();
            for(int i = 0; i<buttons.length; i++){
                for(int j = 0; j<buttons.length;j++){
                    JButton b0 = buttons[i][j];
                    grid.add(b0);
                    grid.repaint();
                }
            }
        }
        
        public static void finish(String[] filepath, String path, String[][] data, String[][] colors) throws FileNotFoundException{
            JFrame frame = new JFrame("Highscore anlegen");
            JPanel mainPanel = new JPanel(new BorderLayout());
            JLabel text = new JLabel("Geben Sie unten Ihren Namen an um sich in der Highscore Liste einzutragen.");
            mainPanel.add(text, BorderLayout.NORTH);
            JTextField field = new JTextField(20);
            mainPanel.add(field, BorderLayout.CENTER);
            JButton b = new JButton("In der Highscore Liste eintragen");
            mainPanel.add(b, BorderLayout.SOUTH);
            frame.setVisible(true);
            frame.setSize(600,600);
            frame.add(mainPanel);
            b.addActionListener(e -> {
                String username = field.getText();
                try {
                    HighscoreGUI.newRecord(path, username);
                    frame.dispose();
                } catch (FileNotFoundException e1) {
                    // TODO Auto-generated catch block
                    e1.printStackTrace();
                }
                MenuGUI.getPath(filepath);
            });
            filepath[0] = "";
            filepath[1] = "";
        }
}
